  
  class BuyableAttrPerksConfig {
    // Skip items which price above this value
    public static func AttributeCost() -> Int32 = 25000
    public static func PerkCost() -> Int32 = 10000
  }
  
  @replaceMethod(PlayerDevelopmentData)
  private final const func CanAttributeBeBought(type: gamedataStatType) -> Bool {
    let currVal: Int32;
    let enoughPoints: Bool;
    let maxLvlNotReached: Bool;
    let objectID: StatsObjectID = Cast(this.m_owner.GetEntityID());
    let dIndex: Int32 = this.GetDevPointsIndex(gamedataDevelopmentPointType.Attribute);
    if dIndex < 0 {
      LogDM("CanAttributeBeBought(): Attribute development points don\'t exist! Big error! Returns false!");
      return false;
    };
    currVal = Cast(GameInstance.GetStatsSystem(this.m_owner.GetGame()).GetStatValue(objectID, type));
    enoughPoints = this.m_devPoints[dIndex].unspent >= this.GetAttributeNextLevelCost(type);
    maxLvlNotReached = this.GetAttributeDevCap(type) > currVal;
    
    // inserted
    if !enoughPoints && maxLvlNotReached {
      let difference : Int32 = this.GetAttributeNextLevelCost(type) - this.m_devPoints[dIndex].unspent;
      if (this.CheckLocalPlayerMoney__BuyableAttrPerks(BuyableAttrPerksConfig.AttributeCost() * difference)) {
        enoughPoints = true;
      }
    }
    
    if enoughPoints && maxLvlNotReached {
      return true;
    };
    if !enoughPoints {
      LogDM("CanAttributeBeBought(): Not enough dev points!");
    } else {
      LogDM("CanAttributeBeBought(): Attribute " + EnumValueToString("gamedataStatType", Cast(EnumInt(type))) + " has reached max level!");
    };
    return false;
  }
  
  @replaceMethod(PlayerDevelopmentData)
  public final const func BuyAttribute(type: gamedataStatType) -> Bool {
    let cost: Int32;
    let dIndex: Int32 = this.GetDevPointsIndex(gamedataDevelopmentPointType.Attribute);
    if dIndex < 0 {
      return false;
    };
    if !this.CanAttributeBeBought(type) {
      return false;
    };
    cost = this.GetAttributeNextLevelCost(type);
    this.ModifyAttribute(type, 1.00);
    
    // inserted
    if (this.m_devPoints[dIndex].unspent < cost) {
      let difference : Int32 = cost - this.m_devPoints[dIndex].unspent;
      this.PayLocalPlayerMoney__BuyableAttrPerks(BuyableAttrPerksConfig.AttributeCost() * difference);
      this.m_devPoints[dIndex].unspent = cost;
    }
    
    this.m_devPoints[dIndex].unspent -= cost;
    this.m_devPoints[dIndex].spent += cost;
    return true;
  }
  
  @replaceMethod(PlayerDevelopmentData)
  private final const func CanPerkBeBought(perkType: gamedataPerkType) -> Bool {
    let pIndex: Int32;
    let primDevIndex: Int32;
    let pAreaIndex: Int32 = this.GetPerkAreaIndex(this.GetPerkAreaFromPerk(perkType));
    if pAreaIndex < 0 {
      return false;
    };
    pIndex = this.GetPerkIndex(pAreaIndex, perkType);
    if pIndex >= 0 && this.GetPerkMaxLevel(perkType) <= this.m_perkAreas[pAreaIndex].boughtPerks[pIndex].currLevel {
      LogDM("CanPerkBeBought(): Maximum level reached !");
      return false;
    };
    primDevIndex = this.GetDevPointsIndex(gamedataDevelopmentPointType.Primary);
    if primDevIndex < 0 {
      LogDM(" CanPerkBeBought(): Development points not found!");
      return false;
    };
    if this.m_devPoints[primDevIndex].unspent <= 0 && !this.CheckLocalPlayerMoney__BuyableAttrPerks(BuyableAttrPerksConfig.PerkCost()) { // modified
      LogDM("CanPerkBeBought(): Not enough development points!!");
      return false;
    };
    return true;
  }
  
  @replaceMethod(PlayerDevelopmentData)
  public final const func BuyPerk(perkType: gamedataPerkType) -> Bool {
    let canBeBought: Bool;
    let newPerk: SPerk;
    let pIndex: Int32;
    let profIndex: Int32;
    let pAreaIndex: Int32 = this.GetPerkAreaIndex(this.GetPerkAreaFromPerk(perkType));
    if pAreaIndex < 0 {
      return false;
    };
    pIndex = this.GetPerkIndex(perkType);
    canBeBought = this.CanPerkBeBought(perkType);
    if pIndex < 0 && canBeBought {
      newPerk = this.InitializePerk(perkType);
      ArrayPush(this.m_perkAreas[pAreaIndex].boughtPerks, newPerk);
      pIndex = this.GetPerkIndex(perkType);
    };
    if !this.IsPerkMaxLevel(perkType) && canBeBought {
      profIndex = this.GetProficiencyIndexFromPerkArea(this.m_perkAreas[pAreaIndex].type);
      this.DeactivatePerkLevelData(pAreaIndex, pIndex);
      this.IncreasePerkLevel(pAreaIndex, pIndex);
      this.ActivatePerkLevelData(pAreaIndex, pIndex);
      
      // inserted
      if (this.m_devPoints[this.GetDevPointsIndex(gamedataDevelopmentPointType.Primary)].unspent <= 0) {
        this.PayLocalPlayerMoney__BuyableAttrPerks(BuyableAttrPerksConfig.PerkCost());
        this.m_devPoints[this.GetDevPointsIndex(gamedataDevelopmentPointType.Primary)].unspent += 1;
      }
      
      this.SpendDevelopmentPoint(gamedataDevelopmentPointType.Primary);
      this.m_proficiencies[profIndex].spentPerkPoints += 1;
      this.EvaluatePerkAreas(this.m_proficiencies[profIndex].type);
      return true;
    };
    return false;
  }
  
  @addMethod(PlayerDevelopmentData)
  public final func CheckLocalPlayerMoney__BuyableAttrPerks(number : Int32) -> Bool {
    return GameInstance.GetTransactionSystem(this.m_owner.GetGame()).GetItemQuantity(this.m_owner, MarketSystem.Money()) >= number;
  }
  
  @addMethod(PlayerDevelopmentData)
  public final func PayLocalPlayerMoney__BuyableAttrPerks(number : Int32) -> Void {
    GameInstance.GetTransactionSystem(this.m_owner.GetGame()).RemoveItem(this.m_owner, MarketSystem.Money(), number);
  }
  
  @replaceMethod(PlayerDevelopmentDataManager)
  public final func IsPerkUpgradeable(data: ref<BasePerkDisplayData>, opt showNotification: Bool) -> Bool {
    let notificationEvent: ref<UIMenuNotificationEvent>;
    let notificationType: UIMenuNotificationType;
    let isPerkUpgradeable: Bool = true;
    if this.GetPerkPoints() <= 0 {
      notificationType = UIMenuNotificationType.NoPerksPoints;
      isPerkUpgradeable = false;
    };
    if data.m_locked {
      notificationType = UIMenuNotificationType.PerksLocked;
      isPerkUpgradeable = false;
    };
    if data.m_maxLevel > -1 && data.m_level >= data.m_maxLevel {
      notificationType = UIMenuNotificationType.MaxLevelPerks;
      isPerkUpgradeable = false;
    };
    
    // inserted
    if (!isPerkUpgradeable && Equals(notificationType, UIMenuNotificationType.NoPerksPoints) ) {
      if (this.CheckLocalPlayerMoney__BuyableAttrPerks(BuyableAttrPerksConfig.PerkCost())) {
        isPerkUpgradeable = true;
      }
    }

    if !isPerkUpgradeable && showNotification {
      notificationEvent = new UIMenuNotificationEvent();
      notificationEvent.m_notificationType = notificationType;
      GameInstance.GetUISystem(this.m_player.GetGame()).QueueEvent(notificationEvent);
    };
    return isPerkUpgradeable;
  }
  
  @replaceMethod(PlayerDevelopmentDataManager)
  public final func HasAvailableAttributePoints(opt showNotification: Bool) -> Bool {
    let notificationEvent: ref<UIMenuNotificationEvent>;
    let points: Bool = this.GetAttributePoints() > 0 || this.CheckLocalPlayerMoney__BuyableAttrPerks(BuyableAttrPerksConfig.AttributeCost()); // modified
    if !points && showNotification {
      notificationEvent = new UIMenuNotificationEvent();
      notificationEvent.m_notificationType = UIMenuNotificationType.NoAttributePoints;
      GameInstance.GetUISystem(this.m_player.GetGame()).QueueEvent(notificationEvent);
    };
    return points;
  }
  
  @addMethod(PlayerDevelopmentDataManager)
  public final func CheckLocalPlayerMoney__BuyableAttrPerks(number : Int32) -> Bool {
    let devData: ref<PlayerDevelopmentData> = PlayerDevelopmentSystem.GetData(this.m_player);
    return devData.CheckLocalPlayerMoney__BuyableAttrPerks(number);
  }